from __future__ import annotations

from pathlib import Path


class StorageLocation:
    def __init__(
        self, prefix: str, bucket: str, blob_name: str, folders: str, filename: str
    ):
        self._prefix = prefix
        self._bucket = bucket
        self._blob_name = blob_name
        self._folders = folders
        self._filename = filename

    @property
    def filename(self) -> str:
        return self._filename

    @property
    def folders(self) -> str:
        return self._folders

    @property
    def blob_name(self) -> str:
        return self._blob_name

    @property
    def bucket(self) -> str:
        return self._bucket

    @property
    def prefix(self) -> str:
        return self._prefix

    def complete_path(self):
        tail = ""
        if self._blob_name is not None:
            tail = str(self.blob_name)
        return str(self.prefix) + str(self.bucket) + "/" + tail


class StorageLocationBuilder:
    def __init__(self):
        self._prefix = "gs://"
        self._bucket = None
        self._blob_name = None
        self._filename = None
        self._folders = None

    def set_prefix(self, prefix: str) -> StorageLocationBuilder:
        self._prefix = prefix
        return self

    def set_bucket(self, bucket: str) -> StorageLocationBuilder:
        self._bucket = bucket
        return self

    def set_blob_name(self, blob_name: str) -> StorageLocationBuilder:
        self._blob_name = blob_name
        return self

    def build(self) -> StorageLocation:
        # Validating setting
        if self._bucket is None:
            raise ValueError("Bucket not set")

        # Getting filename from blob_name
        if self._blob_name is not None:
            self._folders = self._blob_name
            if "." in Path(self._blob_name).name:
                values = self._blob_name.split("/")
                self._filename = values[-1]
                self._folders = "/".join(values[:-1])

        return StorageLocation(
            prefix=self._prefix,
            bucket=self._bucket,
            folders=self._folders,
            blob_name=self._blob_name,
            filename=self._filename,
        )



from typing import List

from google.cloud import storage
from typing import TextIO, BinaryIO, Union


class StorageConnector:
    @staticmethod
    def upload_from_string(
        data: Union[bytes, str], bucket_name: str, destination_blob_name: str
    ) -> None:
        """
        Uploads data to the specified bucket with the specified blob name

        @param data: data to upload in bytes
        @param bucket_name: the destination bucket name
        @param destination_blob_name: the destination blob name
        @return: None
        """

        storage.Client().bucket(bucket_name=bucket_name).blob(
            blob_name=destination_blob_name
        ).upload_from_string(data=data)

    @staticmethod
    def upload_from_file(
        file_handle: Union[TextIO, BinaryIO],
        bucket_name: str,
        destination_blob_name: str,
    ) -> None:
        """
        Uploads data from filename

        @param source_file_name: the source filename
        @param bucket_name: the destination bucket name
        @param destination_blob_name: the destination blob name
        @return: None
        """
        storage.Client().bucket(bucket_name=bucket_name).blob(
            blob_name=destination_blob_name
        ).upload_from_file(file_handle)

    @staticmethod
    def download_as_bytes(bucket_name: str, source_blob_name: str) -> bytes:
        """
        Returns the content of a blob as bytes

        @param bucket_name: the source bucket name
        @param source_blob_name: the source blob name
        @return: the content of the blob as bytes
        """

        return (
            storage.Client()
            .bucket(bucket_name=bucket_name)
            .blob(blob_name=source_blob_name)
            .download_as_bytes()
        )

    @staticmethod
    def download_as_string(bucket_name: str, source_blob_name: str) -> str:
        """
        Returns the content of a blob as a string

        @param bucket_name: the source bucket name
        @param source_blob_name: the source blob name
        @return: the content of the blob as a string
        """

        return StorageConnector.download_as_bytes(
            bucket_name=bucket_name, source_blob_name=source_blob_name
        ).decode("utf-8")

    @staticmethod
    def download_to_filename(
        filename: str, bucket_name: str, source_blob_name: str
    ) -> None:
        """
        Returns the content of a blob to a filename

        @param filename: the name of the file
        @param bucket_name: the source bucket name
        @param source_blob_name: the source blob name
        @return: the content of the blob as a string
        """

        (
            storage.Client()
            .bucket(bucket_name=bucket_name)
            .blob(blob_name=source_blob_name)
            .download_to_filename(filename=filename)
        )

    @staticmethod
    def exists(bucket_name: str, source_blob_name: str) -> bool:
        """
        Returns True if the blob exists

        @param bucket_name: the source bucket name
        @param source_blob_name: the source blob name
        @return: True if gs://bucket_name/source_blob_name exists
        """

        return (
            storage.Client()
            .bucket(bucket_name=bucket_name)
            .blob(blob_name=source_blob_name)
            .exists()
        )

    @staticmethod
    def list_blobs(
        bucket_name: str, prefix: str = None, delimiter: str = None
    ) -> List[str]:
        """
        Returns the list of the blob names

        @param bucket_name: the source bucket name
        @param prefix: prefix to filter blobs
        @param delimiter: Delimiter, used with ``prefix`` to emulate hierarchy.
        @return: list of blob names that match the arguments
        """

        client = storage.Client()
        blobs = storage.Bucket(client=client, name=bucket_name).list_blobs(
            prefix=prefix, delimiter=delimiter
        )

        blobs_names = []
        for blob in blobs:
            blobs_names.append(blob.name)

        return blobs_names

    @staticmethod
    def copy(
        source_bucket_name: str,
        source_blob_name: str,
        dest_bucket_name: str,
        dest_blob_name: str,
    ) -> None:
        """
        Copies a blob to another location

        @param source_bucket_name: the source bucket name
        @param source_blob_name:  the source blob name
        @param dest_bucket_name: the destination bucket name
        @param dest_blob_name: the destination blob name
        @return: None
        """
        client = storage.Client()
        source_bucket = client.bucket(bucket_name=source_bucket_name)
        source_blob = source_bucket.blob(blob_name=source_blob_name)

        dest_bucket = client.bucket(bucket_name=dest_bucket_name)

        return source_bucket.copy_blob(
            blob=source_blob, destination_bucket=dest_bucket, new_name=dest_blob_name
        )

    @staticmethod
    def delete(bucket_name: str, blob_name: str) -> None:
        """
        Deletes a blob

        @param bucket_name: the source bucket name
        @param blob_name: the source blob name
        @return: None
        """
        return (
            storage.Client()
            .bucket(bucket_name=bucket_name)
            .blob(blob_name=blob_name)
            .delete()
        )



import json

from tempfile import TemporaryFile, NamedTemporaryFile

import numpy as np

from wiser.gcloud.storage.connectors.storage_connector import StorageConnector
from wiser.gcloud.storage.types.storage_location import (
    StorageLocation,
    StorageLocationBuilder,
)
from wiser.core.types.extensions import FileExtension


class Storage:
    def __str__(self):
        return "Google Cloud Storage Handler"

    @staticmethod
    def get(location: StorageLocation = None):
        if location.blob_name is None:
            raise ValueError("No blob name given")

        if location.filename.endswith(FileExtension.NUMPY):
            tmp_file = NamedTemporaryFile()
            StorageConnector.download_to_filename(
                filename=tmp_file.name,
                bucket_name=location.bucket,
                source_blob_name=location.blob_name,
            )
            tmp_file.seek(0)
            data = np.load(tmp_file)
            tmp_file.close()
            return data

        elif location.filename.endswith(
            FileExtension.JPG
        ) or location.filename.endswith(FileExtension.PNG):
            return StorageConnector.download_as_bytes(
                bucket_name=location.bucket, source_blob_name=location.blob_name
            )

        elif location.filename.endswith(FileExtension.JSON):
            data = StorageConnector.download_as_string(
                bucket_name=location.bucket, source_blob_name=location.blob_name
            )
            return json.loads(data)
        elif location.filename.endswith(FileExtension.TEXT):
            data = StorageConnector.download_as_string(
                bucket_name=location.bucket, source_blob_name=location.blob_name
            )
            return data
        elif location.filename.endswith(FileExtension.PDF):
            data = StorageConnector.download_as_bytes(
                bucket_name=location.bucket, source_blob_name=location.blob_name
            )
            return data
        else:
            NotImplementedError("File extension not managed")
            return

    @staticmethod
    def save(obj, location: StorageLocation = None):
        if location.filename.endswith(FileExtension.NUMPY):
            tmp_file = TemporaryFile()
            np.save(tmp_file, obj)
            tmp_file.seek(0)
            StorageConnector.upload_from_file(
                tmp_file,
                bucket_name=location.bucket,
                destination_blob_name=location.blob_name,
            )
            tmp_file.close()

        elif location.filename.endswith(
            FileExtension.JPG
        ) or location.filename.endswith(FileExtension.PNG):
            tmp_file = NamedTemporaryFile()
            tmp_file.name = location.filename
            obj.save(tmp_file)
            tmp_file.seek(0)
            StorageConnector.upload_from_file(
                tmp_file,
                bucket_name=location.bucket,
                destination_blob_name=location.blob_name,
            )
            tmp_file.close()

        elif location.filename.endswith(FileExtension.TEXT):
            data = obj
            StorageConnector.upload_from_string(
                data=data,
                bucket_name=location.bucket,
                destination_blob_name=location.blob_name,
            )
        elif location.filename.endswith(FileExtension.JSON):
            data = json.dumps(obj=obj, sort_keys=True, indent=4, ensure_ascii=False)
            StorageConnector.upload_from_string(
                data=data,
                bucket_name=location.bucket,
                destination_blob_name=location.blob_name,
            )
        elif location.filename.endswith(FileExtension.PDF):
            with open(obj, "rb") as f:
                StorageConnector.upload_from_file(
                    file_handle=f,
                    bucket_name=location.bucket,
                    destination_blob_name=location.blob_name,
                )
        else:
            NotImplementedError("File extension not managed")
            return

    @staticmethod
    def exists(location: StorageLocation = None):
        return StorageConnector.exists(
            bucket_name=location.bucket, source_blob_name=location.blob_name
        )

    @staticmethod
    def get_list_content(location: StorageLocation = None) -> [StorageLocation]:
        blobs = StorageConnector.list_blobs(
            bucket_name=location.bucket, prefix=location.folders
        )

        locations_list = []
        for blob_name in blobs:
            if blob_name == location.folders:
                # blob is the folder, not a file
                continue
            base_location = (
                StorageLocationBuilder()
                .set_bucket(bucket=location.bucket)
                .set_blob_name(blob_name=blob_name)
                .build()
            )
            locations_list.append(base_location)

        return locations_list

    @staticmethod
    def move(
        source_location: StorageLocation = None,
        dest_location: StorageLocation = None,
    ):
        StorageConnector.copy(
            source_bucket_name=source_location.bucket,
            source_blob_name=source_location.blob_name,
            dest_bucket_name=dest_location.bucket,
            dest_blob_name=dest_location.blob_name,
        )
        return StorageConnector.delete(
            bucket_name=source_location.bucket,
            blob_name=source_location.blob_name,
        )


try:
    import pkg_resources

    pkg_resources.declare_namespace(__name__)
except ImportError:
    import pkgutil

    __path__ = pkgutil.extend_path(__path__, __name__)

from wiser.gcloud.storage.services import Storage
from wiser.gcloud.storage.connectors import StorageConnector
from wiser.gcloud.storage.types import StorageLocationBuilder, StorageLocation

__all__ = ["Storage", "StorageLocationBuilder", "StorageLocation", "StorageConnector"]



import setuptools
from pathlib import Path

CURRENT_DIR = Path(__file__).parent

# Package metadata
name = "wiser-gcloud-storage"
version = "0.0.4"
author = "Nicola Massarenti"
author_email = "nicola.massarenti@gmail.com"
description = "Google Cloud Storage APIs for wiser"

# Requirements, dependencies and namespaces
extra_requirements = dict()
dependencies = ["google-cloud-storage"]
# Only include packages under the 'wiser' namespace. Do not include tests,
# benchmarks, etc.
packages = [
    package for package in setuptools.find_packages() if package.startswith("wiser")
]
namespaces = ["wiser"]
if "wiser.gcloud" in packages:
    namespaces.append("wiser.gcloud")
if "wiser.gcloud.storage" in packages:
    namespaces.append("wiser.gcloud.storage")

# Setup
setuptools.setup(
    name=name,
    version=version,
    author=author,
    author_email=author_email,
    description=description,
    long_description=None,
    long_description_content_type="text/markdown",
    url="https://github.com/audax-tech/wiser-gcloud-storage",
    project_urls={
        "Bug Tracker": "https://github.com/audax-tech/wiser-gcloud-storage/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=dependencies,
    extras_require=extra_requirements,
    namespace_packages=namespaces,
    packages=packages,
    python_requires=">=3.8",
    platforms="Posix; MacOS X; Windows",
    include_package_data=True,
    zip_safe=False,
)
